@@ -1,17 +1,22 @@
 package Random::Simple;
-BEGIN { our $VERSION = 0.03 }
+
+our $VERSION = '0.06';
+our $debug   = 0;
+
+use strict;
+use warnings;
 
 #############################################################
 
 =encoding utf8
 
-=head1 Random::Simple
+=head1 NAME
 
-Random::Simple - Simple, usable, real world random numbers
+Random::Simple - Generate good random numbers in a user consumable way.
 
 =head2 Why Random::Simple?
 
-To make generating random numbers as easy as possible and in a manner that
+To make generating random numbers as easy as possible I<and> in a manner that
 you can use in real code. Generate "good" random numbers without having to
 think about it.
 
@@ -22,6 +27,17 @@ think about it.
     my $integer = random_int($min, $max); # inclusive
     my $bytes   = random_bytes($count);
 
+=head2 Methodology
+
+Perl's internal C<rand()> function uses C<drand48()> which is an older PRNG,
+and may have limitations. `Random::Simple` uses PCG which is: modern, simple,
+well vetted, and fast.
+
+C<Random::Simple> is automatically seeded with high quality entropy directly
+from your OS. On Linux this is C</dev/urandom> and on Windows it uses
+CryptGenRandom. You will get statistically unique random numbers
+automatically.
+
 =head2 See also
 
 =over
@@ -89,29 +105,38 @@ sub seed {
 
 # Randomly seed the PRNG and warmup
 sub seed_with_random {
-	my $seed1, $seed2;
+	my ($seed1, $seed2);
 
 	if (-r "/dev/urandom") {
 		open(my $FH, "<", "/dev/urandom");
-		my $ok = read($FH, $bytes, 16);
+		my $ok = sysread($FH, my $bytes, 16);
 
-		# Build 2x 64bit unsigned ints from the raw bytes
-		$seed1 = unpack("Q", substr($bytes, 0, 8));
-		$seed2 = unpack("Q", substr($bytes, 8, 8));
+		my ($high, $low);
+
+		#my $has_64bit = ($Config{uvsize} == 8);
+
+		# Build the first 64bit seed manually
+		# Cannot use Q because it doesn't exist on 32bit Perls
+		$high  = unpack("L", substr($bytes, 0, 4));
+		$low   = unpack("L", substr($bytes, 4, 4));
+		$seed1 = ($high << 32) | $low;
+
+		# Build the second 64bit seed
+		$high  = unpack("L", substr($bytes, 8, 4));
+		$low   = unpack("L", substr($bytes, 12, 4));
+		$seed2 = ($high << 32) | $low;
 
 		close $FH;
 	} else {
 		# FIXME: Use real entropy this is just a proof of concept
-		$seed1 = int(rand() * (2**64) -1);
-		$seed2 = int(rand() * (2**64) -1);
+		$seed1 = perl_rand64();
+		$seed2 = perl_rand64();
 	}
 
-	if ($DEBUG) {
-		print "SEEDING RANDOMLY\n";
+	if ($debug) {
+		print "SEEDING RANDOMLY: $seed1 / $seed2\n";
 	}
 
-	#print("XXX: $seed1, $seed2\n");
-
 	_seed($seed1, $seed2);
 
 	$has_been_seeded = 1;
@@ -119,18 +144,27 @@ sub seed_with_random {
 	warmup(32);
 }
 
+sub perl_rand64 {
+	my $high = rand(2**32 - 1);
+	my $low  = rand(2**32 - 1);
+
+	my $ret = ($high << 32 | $low);
+
+	return $ret;
+}
+
 sub random_bytes {
 	my $num = shift();
 
 	if (!$has_been_seeded) { seed_with_random(); }
 
-	my $octets_needed = $num / 8;
+	my $octets_needed = $num / 4;
 
 	my $ret = "";
 	for (my $i = 0; $i < $octets_needed; $i++) {
-		my $num = _rand64();
+		my $num = _rand32();
 
-		$ret .= pack("Q", $num);
+		$ret .= pack("L", $num);
 	}
 
 	$ret = substr($ret, 0, $num);
@@ -145,11 +179,8 @@ sub random_int {
 
 	if ($max < $min) { die("Max can't be less than min"); }
 
-	# FIXME: This is modulus and biased... fix later
-	my $range  = $max - $min + 1; # +1 makes it inclusive of $min AND $max
-	my $num    = _rand64();
-	my $ret    = $num % $range;
-	# Add back the offset
+	my $range = $max - $min + 1; # +1 makes it inclusive
+	my $ret   = _bounded_rand($range);
 	$ret      += $min;
 
 	return $ret;
